/* Testing Program */

#include "quickreturn.h"

int main()
{
   class CQuickReturn qr;
   double output, range_max, range_min;
   double complex cmplx;
   class CPlot plot1, plot2, plot3, plot4;
   array double forces[12] = {0};
   int numpoints = 360;

   bool unit = SI;
   double r1 = 0.025, r2 = 0.010, r4 = 0.065, r5 = 0.030, r7 = 0.050;
   double theta1 = M_PI/2;
   double theta2 = -30*M_PI_180;
   double omega2 = -15.0;
   double rg2 = 0.0125, rg4 = 0.0275, rg5 = 0.0250;
   double delta2 = 30*M_PI/180, delta4 = 15*M_PI/180, delta5 = 30*M_PI/180;
   double ig2 = 0.012, ig4 = 0.119, ig5 = 0.038;
   double m2 = 0.8, m3 = 0.3, m4 = 2.4, m5 = 1.4, m6 = 0.3;
   double fl = -100;

   //bool unit = USC;
   //double r1 = 0.0820, r2 = 0.0328, r4 = 0.2132, r5 = 0.0984, r7 = 0.164;
   //double theta1 = M_PI/2;
   //double theta2 = -30*M_PI_180;
   //double omega2 = -15.0;
   //double rg2 = 0.04101, rg4 = 0.09022, rg5 = 0.08202;
   //double delta2 = 30*M_PI/180, delta4 = 15*M_PI/180, delta5 = 30*M_PI/180;
   //double ig2 = 0.00885, ig4 = 0.08777, ig5 = 0.02803;
   //double m2 = 1.764, m3 = 0.661, m4 = 5.291, m5 = 3.086, m6 = 0.661;
   //double fl = -22.48;

   qr.uscUnit(unit);
   qr.setLinks(r1, r2, r4, r5, r7, theta1);
   qr.setAngVel(omega2);
   qr.setGravityCenter(rg2, rg4, rg5, delta2, delta4, delta5);
   qr.setInertia(ig2, ig4, ig5);
   qr.setMass(m2, m3, m4, m5, m6);
   qr.setForce(fl);
   qr.setNumPoints(numpoints);

/* Calculations */
   output = qr.sliderPos(theta2);
   printf("slide position = %f\n", output);

   qr.sliderRange(range_max, range_min);
   printf("Max = %f, Min = %f\n", range_max, range_min);

   output = qr.sliderVel(theta2);
   printf("slide velocity = %f\n", output);

   output = qr.sliderAccel(theta2);
   printf("slide acceleration = %f\n", output);

   output = qr.getRequiredTorque(theta2);
   printf("required Torque = %f\n", output);

   output = qr.getAngPos(theta2, QR_LINK_4);
   printf("theta 4 = %f\n", output);

   output = qr.getAngPos(theta2, QR_LINK_5);
   printf("theta 5 = %f\n", output);

   output = qr.getAngVel(theta2, QR_LINK_4);
   printf("omega 4 = %f\n", output);

   output = qr.getAngVel(theta2, QR_LINK_5);
   printf("omega 5 = %f\n", output);

   output = qr.getAngAccel(theta2, QR_LINK_4);
   printf("alpha 4 = %f\n", output);

   output = qr.getAngAccel(theta2, QR_LINK_5);
   printf("alpha 5 = %f\n", output);

   cmplx = qr.getPointPos(theta2, QR_POINT_B);
   printf("point B x = %f, point B y = %f\n", real(cmplx), imag(cmplx));

   cmplx = qr.getPointVel(theta2, QR_POINT_A);
   printf("Vel point A x = %f, Vel point A y = %f\n", real(cmplx), imag(cmplx));

   cmplx = qr.getPointVel(theta2, QR_POINT_B);
   printf("Vel point B x = %f, Vel point B y = %f\n", real(cmplx), imag(cmplx));

   cmplx = qr.getPointAccel(theta2, QR_POINT_A);
   printf("Accel point A x = %f, Accel point A y = %f\n", real(cmplx), imag(cmplx));

   cmplx = qr.getPointAccel(theta2, QR_POINT_B);
   printf("Accel point B x = %f, Accel point B y = %f\n", real(cmplx), imag(cmplx));

   qr.getForces(theta2, forces);
   printf("Force Name    Value (N)\n");
   printf("F12x          %f\n", forces[0]);
   printf("F12y          %f\n", forces[1]);
   printf("F23x          %f\n", forces[2]);
   printf("F23y          %f\n", forces[3]);
   printf("F14x          %f\n", forces[4]);
   printf("F14y          %f\n", forces[5]);
   printf("F34           %f\n", forces[6]);
   printf("F45x          %f\n", forces[7]);
   printf("F45y          %f\n", forces[8]);
   printf("F56x          %f\n", forces[9]);
   printf("F56y          %f\n", forces[10]);
   printf("F16y          %f\n", forces[11]);
   printf("\n");

   qr.plotSliderPos(&plot1);
   qr.plotSliderVel(&plot2);
   qr.plotSliderAccel(&plot3);
   //qr.plotAngPos(&plot);
   //qr.plotAngVel(&plot);
   //qr.plotAngAccel(&plot);
   //qr.plotCGaccel(&plot);  
   //qr.plotForce(MAG_F34 | F16Y, &plot4);
   //qr.plotTorque(&plot);

   qr.setNumPoints(50);
   qr.displayPosition(theta2, QANIMATE_OUTPUTTYPE_DISPLAY);
   //qr.displayPosition(theta2, QANIMATE_OUTPUTTYPE_STREAM);
   //qr.displayPosition(theta2, QANIMATE_OUTPUTTYPE_FILE, "position.qnm");
   qr.animation(QANIMATE_OUTPUTTYPE_DISPLAY);
   //qr.animation(QANIMATE_OUTPUTTYPE_STREAM);
   //qr.animation(QANIMATE_OUTPUTTYPE_FILE, "animation.qnm");
   printf("Done!\n");
   return 0;
}
